@@ -2,7 +2,7 @@ import logging
 from pythonagent.lib import get_ident
 import ctypes
 import os
-
+import time
 from pythonagent.agent.probes.havoc.havoc_manager import NDNetHavocMonitor, NDHavocException
 
 from pythonagent.agent.internal import udp
@@ -66,8 +66,8 @@ class Agent(object):
         self.last_forced_snapshot = 0
         self.lib = None
 
-        if "CAV_APP_AGENT_Env" in os.environ:
-            if os.environ['CAV_APP_AGENT_Env'] == "AWS_LAMBDA":
+        if "CAV_APP_AGENT_ENV" in os.environ:
+            if os.environ['CAV_APP_AGENT_ENV'] == "AWS_LAMBDA":
                 self.cav_env = "AWS_LAMBDA"
             else:
                 self.cav_env = "NATIVE"
@@ -108,6 +108,8 @@ class Agent(object):
 
         self.havoc_monitor = NDNetHavocMonitor.get_instance()
 
+        self.application_loggers = set()
+
     def start(self):
         path = '/usr/local/lib/libndsdk_py.so'
 
@@ -147,26 +149,26 @@ class Agent(object):
         self.lib = ctypes.cdll.LoadLibrary(lib_path)
         # self.lib.nd_init2.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
         self.lib.nd_method_entry.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
-        self.lib.nd_method_exit.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
+        self.lib.nd_method_exit.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p))]
 
         self.lib.nd_bt_begin.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
         self.lib.nd_bt_begin.restype = ctypes.c_void_p
 
-        self.lib.nd_bt_end.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
+        self.lib.nd_bt_end.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p))]
         self.lib.nd_bt_end.restype = ctypes.c_int
 
         self.lib.nd_bt_store.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
 
-        self.lib.nd_ip_db_callout_begin.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p]
+        self.lib.nd_ip_db_callout_begin.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p,ctypes.c_char_p]
         self.lib.nd_ip_db_callout_begin.restype = ctypes.c_void_p
 
-        self.lib.nd_ip_db_callout_end.argtypes = [ctypes.c_void_p, ctypes.c_void_p]
+        self.lib.nd_ip_db_callout_end.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p))]
         self.lib.nd_ip_db_callout_end.restype = ctypes.c_int
 
         self.lib.nd_ip_http_callout_begin.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p]
         self.lib.nd_ip_http_callout_begin.restype = ctypes.c_void_p
 
-        self.lib.nd_ip_http_callout_end.argtypes = [ctypes.c_void_p, ctypes.c_void_p]
+        self.lib.nd_ip_http_callout_end.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p))]
         self.lib.nd_ip_http_callout_end.restype = ctypes.c_int
 
         self.lib.sdk_getTopoFpStr.argtypes = [ctypes.c_void_p]
@@ -187,9 +189,13 @@ class Agent(object):
         ai_end_ftype = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.c_char_p)
         self.c_endaisession = ai_end_ftype(self.endaisessioncc)
 
-        CB_FTYPE_DOUBLE_DOUBLE = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int,
-                                                  ctypes.c_int)
-        self.c_startad = CB_FTYPE_DOUBLE_DOUBLE(self.thd_funct)
+        ftype_autodiscovery = ctypes.CFUNCTYPE(ctypes.c_void_p,  # Return type 
+                                               ctypes.c_char_p,  # methodFiltersPatternList
+                                               ctypes.c_char_p,  # classFiltersPatternList
+                                               ctypes.c_int,     # methodListLength
+                                               ctypes.c_int)     # classListLength
+
+        self.autodiscovery_c = ftype_autodiscovery(self.autodiscovery_py)
 
         thd_ftype = ctypes.CFUNCTYPE(ctypes.c_char_p)
         self.c_thd = thd_ftype(self.thd)
@@ -225,7 +231,8 @@ class Agent(object):
         return self.lib
 
     def sdk_getNDSessionCookie(self, bt):
-        return self.lib.sdk_getNDSessionCookie(bt).decode()
+        nd_cookie_value =  self.lib.sdk_getNDSessionCookie(bt)
+        return nd_cookie_value
 
     def exception_dump_native(self, bt, mpp, starttime, excptnclsname, excptnmsg, throwingclsname,
                         throwingmtdname, excptncause, excptnlineno, stacktrace):
@@ -304,14 +311,15 @@ class Agent(object):
 
     def startaisession(self, istart, startTime, duration, sname):
         print("session status \n", istart)
-        print(" IN startaisession", sname.decode('ascii'))
-        wsgi.aiflagset(istart, startTime, duration, sname)
-        #pathd = "/opt/cavisson/netdiagnostics/logs/"
-        nd_home = os.environ.get('ND_HOME')
-        pathd = nd_home + '/python/logs/'
-        filenameai1 = str(self.sname.decode('ascii')) + ".txt"
-        aifile_to_open_to_d = os.path.join(pathd, filenameai1)
-        self.lib.handleUploadDownload(bytes(aifile_to_open_to_d, 'utf-8'))
+        #print(" IN startaisession", sname.decode('ascii'))
+        if sname:
+            wsgi.aiflagset(istart, startTime, duration, sname)
+            #pathd = "/opt/cavisson/netdiagnostics/logs/"
+            nd_home = os.environ.get('ND_HOME')
+            pathd = nd_home + '/python/logs/'
+            filenameai1 = str(self.sname.decode('ascii')) + ".txt"
+            aifile_to_open_to_d = os.path.join(pathd, filenameai1)
+            self.lib.handleUploadDownload(bytes(aifile_to_open_to_d, 'utf-8'))
 
     def sdk_getTopoFpStr(self, bt):
         topo = None
@@ -329,7 +337,7 @@ class Agent(object):
         #print("**** callback registered for NV cookie ****")
 
     def ad_wrapper(self):
-        self.lib.register_AD_callback(self.c_startad)
+        self.lib.register_AD_callback(self.autodiscovery_c)
 
     def thd_wrapper(self):
         self.lib.register_thread_dump_callback(self.c_thd)
@@ -339,8 +347,19 @@ class Agent(object):
         print("INSIDE THD")
         # dumpstacks()
 
-    def thd_funct(self, a, b, c, d):
-        find("/home/cavisson/shop/my-shop/FlaskwithMysqlapp")
+    def autodiscovery_py(self, a, b, c, d):
+        #print("\n\n\nAD BEGIN\n\n\n")
+        #print("a {} b {} c {} d{}".format(a,b,c,d))
+        start_time = time.time()
+        output_from_ad, num_of_modules, num_of_methods = find()
+        end_time = time.time()
+        total_time = end_time - start_time
+        ad_summary =  "totalTimeTaken={} ms;totalModules={};totalMethods={};\n".format(total_time, num_of_modules, num_of_methods)
+        #print(output_from_ad)
+        self.lib.wraphandleMessageForDiscovery(ctypes.c_char_p(bytes(output_from_ad, "utf-8")))
+        self.lib.wraphandleMessageForDiscovery(ctypes.c_char_p(bytes(ad_summary, "utf-8")))
+        #print("\n\n\nAD END\n\n\n")
+
 
     # @profile
     def sdk_init_native(self):
@@ -403,7 +422,10 @@ class Agent(object):
             except:
                 method_bytes = bytes(method.encode("utf-8"))
             #print("\n\n", method_bytes)
-            self.lib.nd_method_exit(bt, ctypes.c_char_p(method_bytes))
+            str_span_c = ctypes.POINTER(ctypes.c_char_p)()
+            self.lib.nd_method_exit(bt, ctypes.c_char_p(method_bytes), ctypes.byref(str_span_c))
+            span_obj = ctypes.c_char_p.from_buffer(str_span_c)
+            self.dump_span(span_obj)
 
     def method_exit(self, bt, method, status=200, duration=0):
         if self.cav_env == "NATIVE":
@@ -430,6 +452,7 @@ class Agent(object):
 
     # @profile
     def start_business_transaction_native(self, bt_name, correlation_header, nd_cookie, nv_cookie, bt_header_value):
+        self.unset_current_bt()
         if os.environ['nd_init_done'] == '0':
             self.start()
             #print("start_business_transaction ->")
@@ -461,7 +484,8 @@ class Agent(object):
         nv_cookie_bytes = get_utf8_bytes(str(nv_cookie))
         nv_cookie_c = ctypes.c_char_p(nv_cookie_bytes)
 
-        bt_header_value_c = ctypes.c_char_p(None)
+        bt_header_value_bytes = get_utf8_bytes(str(bt_header_value))
+        bt_header_value_c = ctypes.c_char_p(bt_header_value_bytes)
         
         flow_path_id = self.get_transaction_context().flow_path_id
         flow_path_id_bytes = get_utf8_bytes(str(flow_path_id))
@@ -497,9 +521,14 @@ class Agent(object):
         #print("end_business_transaction--> BT: ", bt)
         self.logger.info('At end_business_transaction bt value is {0}'.format(bt))
         statuscode = self.get_current_status_code()
-        rc = self.lib.nd_bt_end(bt, statuscode, ctypes.c_int(0))
+        str_span_c = ctypes.POINTER(ctypes.c_char_p)()
+        rc = self.lib.nd_bt_end(bt, statuscode, ctypes.c_int(0), ctypes.byref(str_span_c))
+
+        span_obj = ctypes.c_char_p.from_buffer(str_span_c)
+        self.dump_span(span_obj)
+
         self.logger.info("nd_bt_end return value:  {0}".format(int(rc)))
-        self.unset_current_bt()
+        #self.unset_current_bt()
         self.unset_current_status_code()
         self.active_bts.discard(bt)
         return int(rc)
@@ -513,6 +542,16 @@ class Agent(object):
         return rc
 
 
+    def dump_span(self, span_obj):
+        if not span_obj:
+            return
+        span_value = span_obj.value.decode()
+        for x in self.application_loggers:
+            #print("\n inside for", x.name, "\n")
+            if x.name != "werkzeug" and not x.name.startswith("pythonagent"):
+                #print("\n inside if", x.name, "\n")
+                x.info(span_value)
+
     #@profile
     def store_business_transaction_native(self, bt, unique_bt_id):
         self.logger.info('lib variable in store business txn: {0}'.format(self.lib))
@@ -534,7 +573,7 @@ class Agent(object):
 
 
     #@profile
-    def db_call_begin_native(self, bt, db_host, db_query):
+    def db_call_begin_native(self, bt, db_host, db_query, query_parameters):
         #db_host_c = bytes(db_host, 'utf-8')
         #db_query_c = bytes(db_query, 'utf-8')
         try:
@@ -544,13 +583,20 @@ class Agent(object):
             db_host_c = bytes(db_host.encode("utf-8"))
             db_query_c = bytes(db_query.encode("utf-8"))
 
-        ip_handle = self.lib.nd_ip_db_callout_begin(bt, ctypes.c_char_p(db_host_c), ctypes.c_char_p(db_query_c))
+        if query_parameters:
+            db_query_paramaters_bytes = get_utf8_bytes(str(query_parameters))
+            db_query_parameters_c = ctypes.c_char_p(db_query_paramaters_bytes)
+        else:
+            db_query_parameters_c = ctypes.c_char_p(None)
+
+        ip_handle = self.lib.nd_ip_db_callout_begin(bt, ctypes.c_char_p(db_host_c), ctypes.c_char_p(db_query_c), db_query_parameters_c)
         return ip_handle
 
 
-    def db_call_begin(self, bt, db_host, db_query):
+    def db_call_begin(self, bt, db_host, db_query, query_parameters):
+        db_host = "NA|" + str(db_host) + "|NA|NA|NA|NA|NA|NA|NA|NA|NA|NA"
         if self.cav_env == "NATIVE":
-            ip_handle = self.db_call_begin_native(bt, db_host, db_query)
+            ip_handle = self.db_call_begin_native(bt, db_host, db_query,query_parameters)
             return ip_handle
         elif self.cav_env == "AWS_LAMBDA":
             udp.db_call_begin(bt, db_host, db_query)
@@ -558,9 +604,11 @@ class Agent(object):
 
     #@profile    #@staticmethod
     def db_call_end_native(self, bt, ip_handle):
-            rc = self.lib.nd_ip_db_callout_end(bt, ip_handle)
-            return int(rc)
-
+        str_span_c = ctypes.POINTER(ctypes.c_char_p)()
+        rc = self.lib.nd_ip_db_callout_end(bt, ip_handle, ctypes.byref(str_span_c))
+        span_obj = ctypes.c_char_p.from_buffer(str_span_c)
+        self.dump_span(span_obj)
+        return int(rc)
 
     def db_call_end(self, bt, ip_handle):
         if self.cav_env == "NATIVE":
@@ -590,7 +638,10 @@ class Agent(object):
 
     #@profile #@staticmethod
     def http_call_end_native(self, bt, ip_handle):
-        rc = self.lib.nd_ip_http_callout_end(bt, ip_handle)
+        str_span_c = ctypes.POINTER(ctypes.c_char_p)()
+        rc = self.lib.nd_ip_http_callout_end(bt, ip_handle, ctypes.byref(str_span_c))
+        span_obj = ctypes.c_char_p.from_buffer(str_span_c)
+        self.dump_span(span_obj)
         return int(rc)
 
     def http_call_end(self, bt, ip_handle):
