@@ -86,8 +86,8 @@ def configure_logging():
 
         agent_env = None
 
-        if "CAV_APP_AGENT_Env" in os.environ:
-            if os.environ['CAV_APP_AGENT_Env'] == "AWS_LAMBDA":
+        if "CAV_APP_AGENT_ENV" in os.environ:
+            if os.environ['CAV_APP_AGENT_ENV'] == "AWS_LAMBDA":
                 agent_env = "AWS_LAMBDA"
             else:
                 agent_env = "NATIVE"
