@@ -8,6 +8,7 @@ import json
 from .udp_message import create_start_transaction_message, create_end_transaction_message
 from .udp_message import create_method_entry_message, create_method_exit_message
 from .udp_message import create_transaction_encode_http_message
+from .udp_message import create_exception_encode_message
 from .udp_message import create_havoc_message
 from .udp_message import create_init_message
 from .udp_message import create_heartbeat_message
@@ -241,6 +242,14 @@ def http_req_resp_wrapper(agent_obj, bt, rbuffer, rtype, statuscode):
     logger.info("transaction_encode_http message {}".format(message))
     agent_obj.udp_connection.send(message)
 
+def exception_dump(agent_obj, bt, mpp, starttime, excptnclsname, excptnmsg, throwingclsname,
+                   throwingmtdname, excptncause, excptnlineno, stacktrace):
+    context = agent_obj.get_transaction_context()
+    message = create_exception_encode_message(context, excptnlineno, excptnclsname, throwingclsname, throwingmtdname,  
+                                              excptnmsg, excptncause, stacktrace, starttime)
+    logger.info("exception_encode message {}".format(message))
+    agent_obj.udp_connection.send(message)
+
 def havoc_message(agent_obj, havoc_header):
     context = agent_obj.get_transaction_context()
     message = create_havoc_message(context, havoc_header)
