@@ -247,7 +247,7 @@ class MethodExitMessage(object):
 
         self.methodName = methodName
         self.backend_header = backend_header
-        self.requestNotificationPhase = "dummy_requestNotificationPhase"
+        self.requestNotificationPhase = ""
         self.statusCode = status
         self.duration = duration
         self.mid = 0  # DUMMY
@@ -273,7 +273,7 @@ class MethodExitMessage(object):
     def encode(self):
         self.len_methodName = len(self.methodName)
         self.len_backend_header = len(self.backend_header)
-        self.len_requestNotificationPhase = len(self.requestNotificationPhase)
+        self.len_requestNotificationPhase = 0
 
         self.header_len = (4                                       # header_len(int)
                            + 4                                     # total_len(int)
