@@ -128,11 +128,11 @@ class ExitCallInterceptor(BaseInterceptor):
             self.agent.method_entry_http_callout(zero_bt, method_name, http_host, url)  # METHOD ENTRY FOR PROXY
             return self.agent.http_call_begin(bt, http_host, url)
 
-    def db_call_begin(self, bt, db_host, db_query):
+    def db_call_begin(self, bt, db_host, db_query, query_parameters):
         """Start an exit call.
         """
         with self.log_exceptions():
-            return self.agent.db_call_begin(bt, db_host, db_query)
+            return self.agent.db_call_begin(bt, db_host, db_query, query_parameters)
 
     def run(self, func, *args, **kwargs):
         """Run the function.  If it raises an exception, end the exit call started from func
