@@ -67,7 +67,7 @@ class Aisession():
         #f.writelines(f"#End_Time {}")
         f.writelines(f"#Total Duration {self.duration}\n")
         p = pstats.Stats(self.prof)
-        client_app_root = os.path.dirname(sys.argv[0])
+        client_app_root = os.path.abspath(os.path.dirname(sys.argv[0]))
         print("application:-",client_app_root)
         methodcountdict = {}
         for k,v in p.stats.items():
