@@ -127,8 +127,9 @@ class WSGIInterceptor(EntryPointInterceptor):
                 nd_cookie_key = self.agent.nd_cookie_key
                 if nd_cookie_key:
                     nd_cookie_value = self.agent.sdk_getNDSessionCookie(bt)
-                    nd_cookie_str = "{}={}".format(nd_cookie_key, nd_cookie_value)
-                    headers.append(('Set-Cookie', nd_cookie_str))
+                    if nd_cookie_value:
+                        nd_cookie_str = "{}={}".format(nd_cookie_key, nd_cookie_value.decode())
+                        headers.append(('Set-Cookie', nd_cookie_str))
 
                 resp_headers = self.parse_headers(headers.copy())
 
