@@ -12,6 +12,8 @@ class LoggingInterceptor(BaseInterceptor):
     LOGGING_ERROR_DISPLAY_NAME = '{logger_name} [{level_name}]'
 
     def _callHandlers(self, callHandlers, logger, record):
+        self.agent.application_loggers.add(logger)
+
         with self.log_exceptions():
             if not record.name.startswith('pythonagent') and self.bt:
                 self.agent.logger.info("error message as {}".format(record.getMessage()))
@@ -73,7 +75,7 @@ class LoggingInterceptor(BaseInterceptor):
 
         display_name = self.LOGGING_ERROR_DISPLAY_NAME.format(logger_name=record.name, level_name=record.levelname)
         #self.bt.add_logged_error(ErrorInfo(msg, display_name, levelno))
-        self.agent.logger.info('Logging error added in logs as message {0} display name as {1} and level of error is (2}'.format(msg,display_name,levelno))
+        self.agent.logger.info('Logging error added in logs as message {0} display name as {1} and level of error is {2}'.format(msg,display_name,levelno))
 
 
 def intercept_logging(agent, mod):
