@@ -53,7 +53,7 @@ class BotocoreClientInterceptor(ExitCallInterceptor):
                     if operation_name is None:
                         operation_name = "default"
 
-                    method_name = "botocore.client." + str(operation_name)
+                    method_name = "botocore.client.ClientCreator._create_api_method"
 
                     query_string = str(table_name)+"."+str(operation_name)
                     start_time = get_current_timestamp_in_ms() 
