@@ -89,8 +89,8 @@ SERVER = myvars['server'].strip('\n')
 LOGS_DIR = ospath+'/python/logs'
 print("LOGS_DIR", LOGS_DIR)
 
-if "CAV_APP_AGENT_LoggingLevel" in os.environ:
-    LOGGING_LEVEL = os.environ.get('CAV_APP_AGENT_LoggingLevel')
+if "CAV_APP_AGENT_LOGGINGLEVEL" in os.environ:
+    LOGGING_LEVEL = os.environ.get('CAV_APP_AGENT_LOGGINGLEVEL')
 else:
     LOGGING_LEVEL = "WARNING"
 
